@@ -1,9 +1,8 @@
-/* Copyright (C) 2000,2004 MySQL AB
+/* Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,7 +30,7 @@ extern "C" {
 #include <thr_lock.h>
 #endif
 
-#include "my_handler.h"
+#include "my_compare.h"
 #include "my_tree.h"
 
 	/* defines used by heap-funktions */
@@ -46,8 +45,8 @@ typedef struct st_heapinfo		/* Struct from heap_info */
   ulong records;			/* Records in database */
   ulong deleted;			/* Deleted records in database */
   ulong max_records;
-  ulong data_length;
-  ulong index_length;
+  ulonglong data_length;
+  ulonglong index_length;
   uint reclength;			/* Length of one record */
   int errkey;
   ulonglong auto_increment;
@@ -135,7 +134,7 @@ typedef struct st_heap_share
   HP_BLOCK block;
   HP_KEYDEF  *keydef;
   ulong min_records,max_records;	/* Params to open */
-  ulong data_length,index_length,max_table_size;
+  ulonglong data_length,index_length,max_table_size;
   uint key_stat_version;                /* version to indicate insert/delete */
   uint records;				/* records */
   uint blength;				/* records rounded up to 2^n */
@@ -187,7 +186,7 @@ typedef struct st_heap_create_info
 {
   uint auto_key;                        /* keynr [1 - maxkey] for auto key */
   uint auto_key_type;
-  ulong max_table_size;
+  ulonglong max_table_size;
   ulonglong auto_increment;
   my_bool with_auto_increment;
 } HP_CREATE_INFO;
