@@ -1,4 +1,5 @@
-/* Copyright (C) 2000 MySQL AB
+/* Copyright (c) 2000-2007 MySQL AB, 2009 Sun Microsystems, Inc.
+   Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /* This file includes constants used with all databases */
 /* Author: Michael Widenius */
@@ -377,8 +378,10 @@ enum ha_base_keytype {
 #define HA_ERR_TABLE_READONLY    161  /* The table is not writable */
 #define HA_ERR_AUTOINC_READ_FAILED 162/* Failed to get the next autoinc value */
 #define HA_ERR_AUTOINC_ERANGE    163  /* Failed to set the row autoinc value */
+#define HA_ERR_TOO_MANY_CONCURRENT_TRXS 164 /*Too many active concurrent transactions */
+/* You must also add numbers and description to extra/perror.c ! */
 
-#define HA_ERR_LAST              163  /*Copy last error nr.*/
+#define HA_ERR_LAST              164  /*Copy last error nr.*/
 /* Add error numbers before HA_ERR_LAST and change it accordingly. */
 #define HA_ERR_ERRORS            (HA_ERR_LAST - HA_ERR_FIRST + 1)
 
