@@ -1679,8 +1679,8 @@ pars_get_lex_chars(
 {
 	int	len;
 	
-	len = pars_sym_tab_global->string_len
-				- pars_sym_tab_global->next_char_pos;
+	len= (uint) (pars_sym_tab_global->string_len
+				- pars_sym_tab_global->next_char_pos);
 	if (len == 0) {
 #ifdef YYDEBUG
 		/* fputs("SQL string ends\n", stderr); */
