@@ -1,9 +1,8 @@
-/* Copyright (C) 2004 MySQL AB
+/* Copyright (c) 2002, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,27 +11,50 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "mysql_priv.h"
 
 #ifdef HAVE_SPATIAL
 
-#define MAX_DIGITS_IN_DOUBLE 16
+/* 
+  exponential notation :
+  1   sign
+  1   number before the decimal point
+  1   decimal point
+  14  number of significant digits (see String::qs_append(double))
+  1   'e' sign
+  1   exponent sign
+  3   exponent digits
+  ==
+  22
+
+  "f" notation :
+  1   optional 0
+  1   sign
+  14  number significant digits (see String::qs_append(double) )
+  1   decimal point
+  ==
+  17
+*/
+
+#define MAX_DIGITS_IN_DOUBLE 22
 
 /***************************** Gis_class_info *******************************/
 
-Geometry::Class_info *Geometry::ci_collection[Geometry::wkb_end+1]=
+String Geometry::bad_geometry_data("Bad object", &my_charset_bin);
+
+Geometry::Class_info *Geometry::ci_collection[Geometry::wkb_last+1]=
 {
   NULL, NULL, NULL, NULL, NULL, NULL, NULL
 };
 
 static Geometry::Class_info **ci_collection_end=
-                                Geometry::ci_collection+Geometry::wkb_end + 1;
+                                Geometry::ci_collection+Geometry::wkb_last + 1;
 
 Geometry::Class_info::Class_info(const char *name, int type_id,
 					 void(*create_func)(void *)):
-  m_name(name, strlen(name)), m_type_id(type_id), m_create_func(create_func)
+  m_name(name, (uint) strlen(name)), m_type_id(type_id), m_create_func(create_func)
 {
   ci_collection[type_id]= this;
 }
@@ -119,24 +141,21 @@ Geometry::Class_info *Geometry::find_class(const char *name, uint32 len)
   return 0;
 }
 
-Geometry *Geometry::create_from_wkb(Geometry_buffer *buffer,
-				    const char *data, uint32 data_len)
+
+Geometry *Geometry::construct(Geometry_buffer *buffer,
+                              const char *data, uint32 data_len)
 {
   uint32 geom_type;
   Geometry *result;
+  char byte_order;
 
-  if (data_len < 1 + 4)
+  if (data_len < SRID_SIZE + WKB_HEADER_SIZE)   // < 4 + (1 + 4)
     return NULL;
-  data++;
-  /*
-    FIXME: check byte ordering
-    Also check if we could replace this with one byte
-  */
-  geom_type= uint4korr(data);
-  data+= 4;
+  byte_order= data[SRID_SIZE];
+  geom_type= uint4korr(data + SRID_SIZE + 1);
   if (!(result= create_by_typeid(buffer, (int) geom_type)))
     return NULL;
-  result->m_data= data;
+  result->m_data= data+ SRID_SIZE + WKB_HEADER_SIZE;
   result->m_data_end= data + data_len;
   return result;
 }
@@ -168,13 +187,74 @@ Geometry *Geometry::create_from_wkt(Geometry_buffer *buffer,
     return NULL;
   if (init_stream)  
   {
-    result->init_from_wkb(wkt->ptr(), wkt->length());
+    result->set_data_ptr(wkt->ptr(), wkt->length());
     result->shift_wkb_header();
   }
   return result;
 }
 
 
+static double wkb_get_double(const char *ptr, Geometry::wkbByteOrder bo)
+{
+  double res;
+  if (bo != Geometry::wkb_xdr)
+  {
+    float8get(res, ptr);
+  }
+  else
+  {
+    char inv_array[8];
+    inv_array[0]= ptr[7];
+    inv_array[1]= ptr[6];
+    inv_array[2]= ptr[5];
+    inv_array[3]= ptr[4];
+    inv_array[4]= ptr[3];
+    inv_array[5]= ptr[2];
+    inv_array[6]= ptr[1];
+    inv_array[7]= ptr[0];
+    float8get(res, inv_array);
+  }
+  return res;
+}
+
+
+static uint32 wkb_get_uint(const char *ptr, Geometry::wkbByteOrder bo)
+{
+  if (bo != Geometry::wkb_xdr)
+    return uint4korr(ptr);
+  /* else */
+  {
+    char inv_array[4];
+    inv_array[0]= ptr[3];
+    inv_array[1]= ptr[2];
+    inv_array[2]= ptr[1];
+    inv_array[3]= ptr[0];
+    return uint4korr(inv_array);
+  }
+}
+
+
+Geometry *Geometry::create_from_wkb(Geometry_buffer *buffer,
+                                    const char *wkb, uint32 len, String *res)
+{
+  uint32 geom_type;
+  Geometry *geom;
+
+  if (len < WKB_HEADER_SIZE)
+    return NULL;
+  geom_type= wkb_get_uint(wkb+1, (wkbByteOrder)wkb[0]);
+  if (!(geom= create_by_typeid(buffer, (int) geom_type)) ||
+      res->reserve(WKB_HEADER_SIZE, 512))
+    return NULL;
+
+  res->q_append((char) wkb_ndr);
+  res->q_append(geom_type);
+
+  return geom->init_from_wkb(wkb + WKB_HEADER_SIZE, len - WKB_HEADER_SIZE,
+                             (wkbByteOrder) wkb[0], res) ? geom : NULL;
+}
+
+
 bool Geometry::envelope(String *result) const
 {
   MBR mbr;
@@ -344,6 +424,20 @@ bool Gis_point::init_from_wkt(Gis_read_stream *trs, String *wkb)
 }
 
 
+uint Gis_point::init_from_wkb(const char *wkb, uint len,
+                              wkbByteOrder bo, String *res)
+{
+  double x, y;
+  if (len < POINT_DATA_SIZE || res->reserve(POINT_DATA_SIZE))
+    return 0;
+  x= wkb_get_double(wkb, bo);
+  y= wkb_get_double(wkb + SIZEOF_STORED_DOUBLE, bo);
+  res->q_append(x);
+  res->q_append(y);
+  return POINT_DATA_SIZE;
+}
+
+
 bool Gis_point::get_data_as_wkt(String *txt, const char **end) const
 {
   double x, y;
@@ -413,6 +507,33 @@ bool Gis_line_string::init_from_wkt(Gis_read_stream *trs, String *wkb)
 }
 
 
+uint Gis_line_string::init_from_wkb(const char *wkb, uint len,
+                                    wkbByteOrder bo, String *res)
+{
+  uint32 n_points, proper_length;
+  const char *wkb_end;
+  Gis_point p;
+
+  if (len < 4)
+    return 0;
+  n_points= wkb_get_uint(wkb, bo);
+  proper_length= 4 + n_points * POINT_DATA_SIZE;
+
+  if (!n_points || len < proper_length || res->reserve(proper_length))
+    return 0;
+
+  res->q_append(n_points);
+  wkb_end= wkb + proper_length;
+  for (wkb+= 4; wkb<wkb_end; wkb+= POINT_DATA_SIZE)
+  {
+    if (!p.init_from_wkb(wkb, POINT_DATA_SIZE, bo, res))
+      return 0;
+  }
+
+  return proper_length;
+}
+
+
 bool Gis_line_string::get_data_as_wkt(String *txt, const char **end) const
 {
   uint32 n_points;
@@ -450,7 +571,7 @@ bool Gis_line_string::get_mbr(MBR *mbr, const char **end) const
 }
 
 
-int Gis_line_string::length(double *len) const
+int Gis_line_string::geom_length(double *len) const
 {
   uint32 n_points;
   double prev_x, prev_y;
@@ -592,7 +713,7 @@ bool Gis_polygon::init_from_wkt(Gis_read_stream *trs, String *wkb)
 	trs->check_next_symbol(')'))
       return 1;
 
-    ls.init_from_wkb(wkb->ptr()+ls_pos, wkb->length()-ls_pos);
+    ls.set_data_ptr(wkb->ptr() + ls_pos, wkb->length() - ls_pos);
     if (ls.is_closed(&closed) || !closed)
     {
       trs->set_error_msg("POLYGON's linear ring isn't closed");
@@ -607,6 +728,45 @@ bool Gis_polygon::init_from_wkt(Gis_read_stream *trs, String *wkb)
 }
 
 
+uint Gis_polygon::init_from_wkb(const char *wkb, uint len, wkbByteOrder bo,
+                                String *res)
+{
+  uint32 n_linear_rings;
+  const char *wkb_orig= wkb;
+
+  if (len < 4)
+    return 0;
+
+  if (!(n_linear_rings= wkb_get_uint(wkb, bo)))
+    return 0;
+
+  if (res->reserve(4, 512))
+    return 0;
+  wkb+= 4;
+  len-= 4;
+  res->q_append(n_linear_rings);
+
+  while (n_linear_rings--)
+  {
+    Gis_line_string ls;
+    uint32 ls_pos= res->length();
+    int ls_len;
+    int closed;
+
+    if (!(ls_len= ls.init_from_wkb(wkb, len, bo, res)))
+      return 0;
+
+    ls.set_data_ptr(res->ptr() + ls_pos, res->length() - ls_pos);
+
+    if (ls.is_closed(&closed) || !closed)
+      return 0;
+    wkb+= ls_len;
+  }
+
+  return (uint) (wkb - wkb_orig);
+}
+
+
 bool Gis_polygon::get_data_as_wkt(String *txt, const char **end) const
 {
   uint32 n_linear_rings;
@@ -777,13 +937,10 @@ int Gis_polygon::interior_ring_n(uint32 num, String *result) const
 int Gis_polygon::centroid_xy(double *x, double *y) const
 {
   uint32 n_linear_rings;
-  double res_area;
-  double res_cx, res_cy;
+  double UNINIT_VAR(res_area);
+  double UNINIT_VAR(res_cx), UNINIT_VAR(res_cy);
   const char *data= m_data;
   bool first_loop= 1;
-  LINT_INIT(res_area);
-  LINT_INIT(res_cx);
-  LINT_INIT(res_cy);
 
   if (no_data(data, 4))
     return 1;
@@ -809,15 +966,14 @@ int Gis_polygon::centroid_xy(double *x, double *y) const
 
     while (--n_points)				// One point is already read
     {
-      double x, y;
-      get_point(&x, &y, data);
+      double tmp_x, tmp_y;
+      get_point(&tmp_x, &tmp_y, data);
       data+= (SIZEOF_STORED_DOUBLE*2);
-      /* QQ: Is the following prev_x+x right ? */
-      cur_area+= (prev_x + x) * (prev_y - y);
-      cur_cx+= x;
-      cur_cy+= y;
-      prev_x= x;
-      prev_y= y;
+      cur_area+= (prev_x + tmp_x) * (prev_y - tmp_y);
+      cur_cx+= tmp_x;
+      cur_cy+= tmp_y;
+      prev_x= tmp_x;
+      prev_y= tmp_y;
     }
     cur_area= fabs(cur_area) / 2;
     cur_cx= cur_cx / (org_n_points - 1);
@@ -895,6 +1051,36 @@ bool Gis_multi_point::init_from_wkt(Gis_read_stream *trs, String *wkb)
 }
 
 
+uint Gis_multi_point::init_from_wkb(const char *wkb, uint len, wkbByteOrder bo,
+                                    String *res)
+{
+  uint32 n_points;
+  uint proper_size;
+  Gis_point p;
+  const char *wkb_end;
+
+  if (len < 4)
+    return 0;
+  n_points= wkb_get_uint(wkb, bo);
+  proper_size= 4 + n_points * (WKB_HEADER_SIZE + POINT_DATA_SIZE);
+ 
+  if (len < proper_size || res->reserve(proper_size))
+    return 0;
+    
+  res->q_append(n_points);
+  wkb_end= wkb + proper_size;
+  for (wkb+=4; wkb < wkb_end; wkb+= (WKB_HEADER_SIZE + POINT_DATA_SIZE))
+  {
+    res->q_append((char)wkb_ndr);
+    res->q_append((uint32)wkb_point);
+    if (!p.init_from_wkb(wkb + WKB_HEADER_SIZE,
+                         POINT_DATA_SIZE, (wkbByteOrder) wkb[0], res))
+      return 0;
+  }
+  return proper_size;
+}
+
+
 bool Gis_multi_point::get_data_as_wkt(String *txt, const char **end) const
 {
   uint32 n_points;
@@ -1004,6 +1190,44 @@ bool Gis_multi_line_string::init_from_wkt(Gis_read_stream *trs, String *wkb)
 }
 
 
+uint Gis_multi_line_string::init_from_wkb(const char *wkb, uint len,
+                                          wkbByteOrder bo, String *res)
+{
+  uint32 n_line_strings;
+  const char *wkb_orig= wkb;
+
+  if (len < 4)
+    return 0;
+  n_line_strings= wkb_get_uint(wkb, bo);
+
+  if (res->reserve(4, 512))
+    return 0;
+  res->q_append(n_line_strings);
+  
+  wkb+= 4;
+  while (n_line_strings--)
+  {
+    Gis_line_string ls;
+    int ls_len;
+
+    if ((len < WKB_HEADER_SIZE) ||
+        res->reserve(WKB_HEADER_SIZE, 512))
+      return 0;
+
+    res->q_append((char) wkb_ndr);
+    res->q_append((uint32) wkb_linestring);
+
+    if (!(ls_len= ls.init_from_wkb(wkb + WKB_HEADER_SIZE, len,
+                                   (wkbByteOrder) wkb[0], res)))
+      return 0;
+    ls_len+= WKB_HEADER_SIZE;;
+    wkb+= ls_len;
+    len-= ls_len;
+  }
+  return (uint) (wkb - wkb_orig);
+}
+
+
 bool Gis_multi_line_string::get_data_as_wkt(String *txt, 
 					     const char **end) const
 {
@@ -1093,7 +1317,7 @@ int Gis_multi_line_string::geometry_n(uint32 num, String *result) const
 }
 
 
-int Gis_multi_line_string::length(double *len) const
+int Gis_multi_line_string::geom_length(double *len) const
 {
   uint32 n_line_strings;
   const char *data= m_data;
@@ -1109,8 +1333,8 @@ int Gis_multi_line_string::length(double *len) const
     double ls_len;
     Gis_line_string ls;
     data+= WKB_HEADER_SIZE;
-    ls.init_from_wkb(data, (uint32) (m_data_end - data));
-    if (ls.length(&ls_len))
+    ls.set_data_ptr(data, (uint32) (m_data_end - data));
+    if (ls.geom_length(&ls_len))
       return 1;
     *len+= ls_len;
     /*
@@ -1138,7 +1362,7 @@ int Gis_multi_line_string::is_closed(int *closed) const
     Gis_line_string ls;
     if (no_data(data, 0))
       return 1;
-    ls.init_from_wkb(data, (uint32) (m_data_end - data));
+    ls.set_data_ptr(data, (uint32) (m_data_end - data));
     if (ls.is_closed(closed))
       return 1;
     if (!*closed)
@@ -1220,6 +1444,43 @@ bool Gis_multi_polygon::init_from_wkt(Gis_read_stream *trs, String *wkb)
 }
 
 
+uint Gis_multi_polygon::init_from_wkb(const char *wkb, uint len,
+                                      wkbByteOrder bo, String *res)
+{
+  uint32 n_poly;
+  const char *wkb_orig= wkb;
+
+  if (len < 4)
+    return 0;
+  n_poly= wkb_get_uint(wkb, bo);
+
+  if (res->reserve(4, 512))
+    return 0;
+  res->q_append(n_poly);
+  
+  wkb+=4;
+  while (n_poly--)
+  {
+    Gis_polygon p;
+    int p_len;
+
+    if (len < WKB_HEADER_SIZE ||
+        res->reserve(WKB_HEADER_SIZE, 512))
+      return 0;
+    res->q_append((char) wkb_ndr);
+    res->q_append((uint32) wkb_polygon);
+
+    if (!(p_len= p.init_from_wkb(wkb + WKB_HEADER_SIZE, len,
+                                 (wkbByteOrder) wkb[0], res)))
+      return 0;
+    p_len+= WKB_HEADER_SIZE;
+    wkb+= p_len;
+    len-= p_len;
+  }
+  return (uint) (wkb - wkb_orig);
+}
+
+
 bool Gis_multi_polygon::get_data_as_wkt(String *txt, const char **end) const
 {
   uint32 n_polygons;
@@ -1356,7 +1617,7 @@ int Gis_multi_polygon::area(double *ar,  const char **end_of_data) const
     Gis_polygon p;
 
     data+= WKB_HEADER_SIZE;
-    p.init_from_wkb(data, (uint32) (m_data_end - data));
+    p.set_data_ptr(data, (uint32) (m_data_end - data));
     if (p.area(&p_area, &data))
       return 1;
     result+= p_area;
@@ -1372,14 +1633,10 @@ int Gis_multi_polygon::centroid(String *result) const
   uint32 n_polygons;
   bool first_loop= 1;
   Gis_polygon p;
-  double res_area, res_cx, res_cy;
+  double UNINIT_VAR(res_area), UNINIT_VAR(res_cx), UNINIT_VAR(res_cy);
   double cur_area, cur_cx, cur_cy;
   const char *data= m_data;
 
-  LINT_INIT(res_area);
-  LINT_INIT(res_cx);
-  LINT_INIT(res_cy);
-
   if (no_data(data, 4))
     return 1;
   n_polygons= uint4korr(data);
@@ -1388,7 +1645,7 @@ int Gis_multi_polygon::centroid(String *result) const
   while (n_polygons--)
   {
     data+= WKB_HEADER_SIZE;
-    p.init_from_wkb(data, (uint32) (m_data_end - data));
+    p.set_data_ptr(data, (uint32) (m_data_end - data));
     if (p.area(&cur_area, &data) ||
 	p.centroid_xy(&cur_cx, &cur_cy))
       return 1;
@@ -1442,7 +1699,7 @@ uint32 Gis_geometry_collection::get_data_size() const
 
     if (!(geom= create_by_typeid(&buffer, wkb_type)))
       return GET_SIZE_ERROR;
-    geom->init_from_wkb(data, (uint) (m_data_end - data));
+    geom->set_data_ptr(data, (uint) (m_data_end - data));
     if ((object_size= geom->get_data_size()) == GET_SIZE_ERROR)
       return GET_SIZE_ERROR;
     data+= object_size;
@@ -1482,6 +1739,48 @@ bool Gis_geometry_collection::init_from_wkt(Gis_read_stream *trs, String *wkb)
 }
 
 
+uint Gis_geometry_collection::init_from_wkb(const char *wkb, uint len,
+                                            wkbByteOrder bo, String *res)
+{
+  uint32 n_geom;
+  const char *wkb_orig= wkb;
+
+  if (len < 4)
+    return 0;
+  n_geom= wkb_get_uint(wkb, bo);
+
+  if (res->reserve(4, 512))
+    return 0;
+  res->q_append(n_geom);
+  
+  wkb+= 4;
+  while (n_geom--)
+  {
+    Geometry_buffer buffer;
+    Geometry *geom;
+    int g_len;
+    uint32 wkb_type;
+
+    if (len < WKB_HEADER_SIZE ||
+        res->reserve(WKB_HEADER_SIZE, 512))
+      return 0;
+
+    res->q_append((char) wkb_ndr);
+    wkb_type= wkb_get_uint(wkb+1, (wkbByteOrder) wkb[0]);
+    res->q_append(wkb_type);
+
+    if (!(geom= create_by_typeid(&buffer, wkb_type)) ||
+        !(g_len= geom->init_from_wkb(wkb + WKB_HEADER_SIZE, len,
+                                     (wkbByteOrder)  wkb[0], res)))
+      return 0;
+    g_len+= WKB_HEADER_SIZE;
+    wkb+= g_len;
+    len-= g_len;
+  }
+  return (uint) (wkb - wkb_orig);
+}
+
+
 bool Gis_geometry_collection::get_data_as_wkt(String *txt,
 					     const char **end) const
 {
@@ -1506,10 +1805,10 @@ bool Gis_geometry_collection::get_data_as_wkt(String *txt,
 
     if (!(geom= create_by_typeid(&buffer, wkb_type)))
       return 1;
-    geom->init_from_wkb(data, (uint) (m_data_end - data));
+    geom->set_data_ptr(data, (uint) (m_data_end - data));
     if (geom->as_wkt(txt, &data))
       return 1;
-    if (txt->append(",", 1, 512))
+    if (txt->append(STRING_WITH_LEN(","), 512))
       return 1;
   }
   txt->length(txt->length() - 1);
@@ -1541,7 +1840,7 @@ bool Gis_geometry_collection::get_mbr(MBR *mbr, const char **end) const
 
     if (!(geom= create_by_typeid(&buffer, wkb_type)))
       return 1;
-    geom->init_from_wkb(data, (uint32) (m_data_end - data));
+    geom->set_data_ptr(data, (uint32) (m_data_end - data));
     if (geom->get_mbr(mbr, &data))
       return 1;
   }
@@ -1582,7 +1881,7 @@ int Gis_geometry_collection::geometry_n(uint32 num, String *result) const
 
     if (!(geom= create_by_typeid(&buffer, wkb_type)))
       return 1;
-    geom->init_from_wkb(data, (uint) (m_data_end - data));
+    geom->set_data_ptr(data, (uint) (m_data_end - data));
     if ((length= geom->get_data_size()) == GET_SIZE_ERROR)
       return 1;
     data+= length;
@@ -1635,7 +1934,7 @@ bool Gis_geometry_collection::dimension(uint32 *res_dim, const char **end) const
     data+= WKB_HEADER_SIZE;
     if (!(geom= create_by_typeid(&buffer, wkb_type)))
       return 1;
-    geom->init_from_wkb(data, (uint32) (m_data_end - data));
+    geom->set_data_ptr(data, (uint32) (m_data_end - data));
     if (geom->dimension(&dim, &end_data))
       return 1;
     set_if_bigger(*res_dim, dim);
