@@ -1,4 +1,5 @@
-/* Copyright (C) 2000-2005 MySQL AB
+/* Copyright (c) 2000-2006 MySQL AB, 2009 Sun Microsystems, Inc.
+   Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /* Support rutiner with are using with dbug */
 
@@ -171,7 +172,7 @@ my_bool check_table_is_closed(const char *name, const char *where)
   char filename[FN_REFLEN];
   LIST *pos;
   DBUG_ENTER("check_table_is_closed");
-
+  pthread_mutex_lock(&THR_LOCK_myisam);
   (void) fn_format(filename,name,"",MI_NAME_IEXT,4+16+32);
   for (pos=myisam_open_list ; pos ; pos=pos->next)
   {
@@ -181,12 +182,14 @@ my_bool check_table_is_closed(const char *name, const char *where)
     {
       if (share->last_version)
       {
-	fprintf(stderr,"Warning:  Table: %s is open on %s\n", name,where);
-	DBUG_PRINT("warning",("Table: %s is open on %s", name,where));
-	DBUG_RETURN(1);
+        fprintf(stderr,"Warning:  Table: %s is open on %s\n", name,where);
+        DBUG_PRINT("warning",("Table: %s is open on %s", name,where));
+        pthread_mutex_unlock(&THR_LOCK_myisam);
+        DBUG_RETURN(1);
       }
     }
   }
+  pthread_mutex_unlock(&THR_LOCK_myisam);
   DBUG_RETURN(0);
 }
 #endif /* EXTRA_DEBUG */
