@@ -1,4 +1,5 @@
-/* Copyright (C) 2000-2006 MySQL AB
+/* Copyright (c) 2000-2008 MySQL AB, 2009 Sun Microsystems, Inc.
+   Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /*
   Functions to handle space-packed-records and blobs
@@ -144,6 +145,29 @@ static int write_dynamic_record(MI_INFO *info, const byte *record,
   DBUG_ENTER("write_dynamic_record");
 
   flag=0;
+
+  /*
+    Check if we have enough room for the new record.
+    First we do simplified check to make usual case faster.
+    Then we do more precise check for the space left.
+    Though it still is not absolutely precise, as
+    we always use MI_MAX_DYN_BLOCK_HEADER while it can be
+    less in the most of the cases.
+  */
+
+  if (unlikely(info->s->base.max_data_file_length -
+               info->state->data_file_length <
+               reclength + MI_MAX_DYN_BLOCK_HEADER))
+  {
+    if (info->s->base.max_data_file_length - info->state->data_file_length +
+        info->state->empty - info->state->del * MI_MAX_DYN_BLOCK_HEADER <
+        reclength + MI_MAX_DYN_BLOCK_HEADER)
+    {
+      my_errno=HA_ERR_RECORD_FILE_FULL;
+      DBUG_RETURN(1);
+    }
+  }
+
   do
   {
     if (_mi_find_writepos(info,reclength,&filepos,&length))
@@ -486,7 +510,7 @@ int _mi_write_part_record(MI_INFO *info,
     }
     length=	  *reclength+head_length;	/* Write only what is needed */
   }
-  DBUG_DUMP("header",(byte*) temp,head_length);
+  DBUG_DUMP("header",(uchar*) temp,head_length);
 
 	/* Make a long block for one write */
   record_end= *record+length-head_length;
@@ -580,6 +604,51 @@ static int update_dynamic_record(MI_INFO *info, my_off_t filepos, byte *record,
   DBUG_ENTER("update_dynamic_record");
 
   flag=block_info.second_read=0;
+  /*
+     Check if we have enough room for the record.
+     First we do simplified check to make usual case faster.
+     Then we do more precise check for the space left.
+     Though it still is not absolutely precise, as
+     we always use MI_MAX_DYN_BLOCK_HEADER while it can be
+     less in the most of the cases.
+  */
+
+  /*
+    compare with just the reclength as we're going
+    to get some space from the old replaced record
+  */
+  if (unlikely(info->s->base.max_data_file_length -
+        info->state->data_file_length < reclength))
+  {
+    /*
+       let's read the old record's block to find out the length of the
+       old record
+    */
+    if ((error=_mi_get_block_info(&block_info,info->dfile,filepos))
+        & (BLOCK_DELETED | BLOCK_ERROR | BLOCK_SYNC_ERROR | BLOCK_FATAL_ERROR))
+    {
+      DBUG_PRINT("error",("Got wrong block info"));
+      if (!(error & BLOCK_FATAL_ERROR))
+        my_errno=HA_ERR_WRONG_IN_RECORD;
+      goto err;
+    }
+
+    /*
+      if new record isn't longer, we can go on safely
+    */
+    if (block_info.rec_len < reclength)
+    {
+      if (info->s->base.max_data_file_length - info->state->data_file_length +
+          info->state->empty - info->state->del * MI_MAX_DYN_BLOCK_HEADER <
+          reclength - block_info.rec_len + MI_MAX_DYN_BLOCK_HEADER)
+      {
+        my_errno=HA_ERR_RECORD_FILE_FULL;
+        goto err;
+      }
+    }
+    block_info.second_read=0;
+  }
+
   while (reclength > 0)
   {
     if (filepos != info->s->state.dellink)
@@ -1069,7 +1138,7 @@ err:
   my_errno= HA_ERR_WRONG_IN_RECORD;
   DBUG_PRINT("error",("to_end: 0x%lx -> 0x%lx  from_end: 0x%lx -> 0x%lx",
 		      (long) to, (long) to_end, (long) from, (long) from_end));
-  DBUG_DUMP("from",(byte*) info->rec_buff,info->s->base.min_pack_length);
+  DBUG_DUMP("from",(uchar*) info->rec_buff,info->s->base.min_pack_length);
   DBUG_RETURN(MY_FILE_ERROR);
 } /* _mi_rec_unpack */
 
@@ -1166,16 +1235,14 @@ void _my_store_blob_length(byte *pos,uint pack_length,uint length)
 int _mi_read_dynamic_record(MI_INFO *info, my_off_t filepos, byte *buf)
 {
   int block_of_record;
-  uint b_type,left_length;
-  byte *to;
+  uint b_type,UNINIT_VAR(left_length);
+  byte *UNINIT_VAR(to);
   MI_BLOCK_INFO block_info;
   File file;
   DBUG_ENTER("mi_read_dynamic_record");
 
   if (filepos != HA_OFFSET_ERROR)
   {
-    LINT_INIT(to);
-    LINT_INIT(left_length);
     file=info->dfile;
     block_of_record= 0;   /* First block of record is numbered as zero. */
     block_info.second_read= 0;
@@ -1438,13 +1505,12 @@ int _mi_read_rnd_dynamic_record(MI_INFO *info, byte *buf,
 {
   int block_of_record, info_read, save_errno;
   uint left_len,b_type;
-  byte *to;
+  byte *UNINIT_VAR(to);
   MI_BLOCK_INFO block_info;
   MYISAM_SHARE *share=info->s;
   DBUG_ENTER("_mi_read_rnd_dynamic_record");
 
   info_read=0;
-  LINT_INIT(to);
 
   if (info->lock_type == F_UNLCK)
   {
@@ -1630,7 +1696,7 @@ uint _mi_get_block_info(MI_BLOCK_INFO *info, File file, my_off_t filepos)
 	sizeof(info->header))
       goto err;
   }
-  DBUG_DUMP("header",(byte*) header,MI_BLOCK_INFO_HEADER_LENGTH);
+  DBUG_DUMP("header",(uchar*) header,MI_BLOCK_INFO_HEADER_LENGTH);
   if (info->second_read)
   {
     if (info->header[0] <= 6 || info->header[0] == 13)
