@@ -449,8 +449,8 @@ int main(int argc, char *argv[])
       bcmp(read_record2,read_record3,reclength))
   {
     printf("Can't find last record\n");
-    DBUG_DUMP("record2",(byte*) read_record2,reclength);
-    DBUG_DUMP("record3",(byte*) read_record3,reclength);
+    DBUG_DUMP("record2",(uchar*) read_record2,reclength);
+    DBUG_DUMP("record3",(uchar*) read_record3,reclength);
     goto end;
   }
   ant=1;
@@ -863,7 +863,6 @@ err:
 static void get_options(int argc, char **argv)
 {
   char *pos,*progname;
-  DEBUGGER_OFF;
 
   progname= argv[0];
 
@@ -976,7 +975,6 @@ static void get_options(int argc, char **argv)
 	     progname);
       exit(0);
     case '#':
-      DEBUGGER_ON;
       DBUG_PUSH (++pos);
       break;
     default:
