@@ -31,7 +31,7 @@ SHM_Transporter::SHM_Transporter(TransporterRegistry &t_reg,
 				 const char *lHostName,
 				 const char *rHostName, 
 				 int r_port,
-				 bool isMgmConnection,
+				 bool isMgmConnection_arg,
 				 NodeId lNodeId,
 				 NodeId rNodeId,
 				 NodeId serverNodeId,
@@ -40,7 +40,7 @@ SHM_Transporter::SHM_Transporter(TransporterRegistry &t_reg,
 				 key_t _shmKey,
 				 Uint32 _shmSize) :
   Transporter(t_reg, tt_SHM_TRANSPORTER,
-	      lHostName, rHostName, r_port, isMgmConnection,
+	      lHostName, rHostName, r_port, isMgmConnection_arg,
 	      lNodeId, rNodeId, serverNodeId,
 	      0, false, checksum, signalId),
   shmKey(_shmKey),
