@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1015,10 +1014,10 @@ NdbOperation::branch_col(Uint32 type,
 			 bool nopad, Uint32 Label){
 
   DBUG_ENTER("NdbOperation::branch_col");
-  DBUG_PRINT("enter", ("type=%u col=%u val=0x%x len=%u label=%u",
-                       type, ColId, val, len, Label));
+  DBUG_PRINT("enter", ("type: %u  col:%u  val: 0x%lx  len: %u  label: %u",
+                       type, ColId, (long) val, len, Label));
   if (val != NULL)
-    DBUG_DUMP("value", (char*)val, len);
+    DBUG_DUMP("value", (uchar*)val, len);
 
   if (initial_interpreterCheck() == -1)
     DBUG_RETURN(-1);
