@@ -19,14 +19,6 @@
 #include <signaldata/NFCompleteRep.hpp>
 #include <signaldata/NodeFailRep.hpp>
 
-static
-void
-require(bool x)
-{
-  if (!x)
-    abort();
-}
-
 SimpleSignal::SimpleSignal(bool dealloc){
   memset(this, 0, sizeof(* this));
   deallocSections = dealloc;
@@ -140,6 +132,8 @@ SignalSender::getNoOfConnectedNodes() const {
 
 SendStatus
 SignalSender::sendSignal(Uint16 nodeId, const SimpleSignal * s){
+  assert(getNodeInfo(nodeId).m_api_reg_conf == true ||
+         s->readSignalNumber() == GSN_API_REGREQ);
   return theFacade->theTransporterRegistry->prepareSend(&s->header,
 							1, // JBB
 							&s->theData[0],
@@ -153,7 +147,10 @@ SignalSender::waitFor(Uint32 timeOutMillis, T & t)
 {
   SimpleSignal * s = t.check(m_jobBuffer);
   if(s != 0){
-    m_usedBuffer.push_back(s);
+    if (m_usedBuffer.push_back(s))
+    {
+      return 0;
+    }
     return s;
   }
   
@@ -168,7 +165,10 @@ SignalSender::waitFor(Uint32 timeOutMillis, T & t)
     
     SimpleSignal * s = t.check(m_jobBuffer);
     if(s != 0){
-      m_usedBuffer.push_back(s);
+      if (m_usedBuffer.push_back(s))
+      {
+        return 0;
+      }
       return s;
     }
     
@@ -181,6 +181,7 @@ SignalSender::waitFor(Uint32 timeOutMillis, T & t)
 
 class WaitForAny {
 public:
+  WaitForAny() {}
   SimpleSignal * check(Vector<SimpleSignal*> & m_jobBuffer){
     if(m_jobBuffer.size() > 0){
       SimpleSignal * s = m_jobBuffer[0];
@@ -200,6 +201,7 @@ SignalSender::waitFor(Uint32 timeOutMillis){
 
 class WaitForNode {
 public:
+  WaitForNode() {}
   Uint32 m_nodeId;
   SimpleSignal * check(Vector<SimpleSignal*> & m_jobBuffer){
     Uint32 len = m_jobBuffer.size();
