@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -98,11 +97,6 @@ OperationTestCase matrix[] = {
   result = NDBT_FAILED; \
   break; }
 
-#define C3(b)  if (!(b)) { \
-  g_err  << "ERR: "<< step->getName() \
-         << " failed on line " << __LINE__ << endl; \
-  abort(); return NDBT_FAILED; }
-
 #define C3(b)  if (!(b)) { \
   g_err  << "ERR: failed on line " << __LINE__ << endl; \
   return NDBT_FAILED; }
