@@ -1,4 +1,19 @@
 #!/bin/sh
+# Copyright (C) 2003-2004, 2006 MySQL AB
+# 
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; version 2 of the License.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 # The default path should be /usr/local
 
 # Get some info from configure
@@ -45,9 +60,11 @@ fi
 mkdir -p $BASE/lib
 
 for i in \
-  libmysql/.libs/libmysqlclient.s{l,o}* \
+  libmysql/.libs/libmysqlclient.so* \
+  libmysql/.libs/libmysqlclient.sl* \
   libmysql/.libs/libmysqlclient*.dylib \
-  libmysql_r/.libs/libmysqlclient_r.s{l,o}* \
+  libmysql_r/.libs/libmysqlclient_r.so* \
+  libmysql_r/.libs/libmysqlclient_r.sl* \
   libmysql_r/.libs/libmysqlclient_r*.dylib
 do
   if [ -f $i ]
