@@ -92,9 +92,14 @@ fix_path pkgincludedir include/mysql include
 
 version='@VERSION@'
 socket='@MYSQL_UNIX_ADDR@'
-port='@MYSQL_TCP_PORT@'
 ldflags='@LDFLAGS@'
 
+if [ @MYSQL_TCP_PORT_DEFAULT@ -eq 0 ]; then
+  port=0
+else
+  port=@MYSQL_TCP_PORT@
+fi
+
 # Create options 
 # We intentionally add a space to the beginning and end of lib strings, simplifies replace later
 libs=" $ldflags -L$pkglibdir -lmysqlclient @ZLIB_DEPS@ @NON_THREADED_LIBS@"
@@ -102,16 +107,29 @@ libs="$libs @openssl_libs@ @STATIC_NSS_FLAGS@ "
 libs_r=" $ldflags -L$pkglibdir -lmysqlclient_r @ZLIB_DEPS@ @LIBS@ @openssl_libs@ "
 embedded_libs=" $ldflags -L$pkglibdir -lmysqld @ZLIB_DEPS@ @LIBS@ @WRAPLIBS@ @innodb_system_libs@ @openssl_libs@ "
 
+if [ -r "$pkglibdir/libmygcc.a" ]; then
+  # When linking against the static library with a different version of GCC
+  # from what was used to compile the library, some symbols may not be defined
+  # automatically.  We package the libmygcc.a from the build host, to provide
+  # definitions for those.  Bugs 4921, 19561, 19817, 21158, etc.
+  libs="$libs -lmygcc "
+  libs_r="$libs_r -lmygcc "
+  embedded_libs="$embedded_libs -lmygcc "
+fi
+
 cflags="-I$pkgincludedir @CFLAGS@ " #note: end space!
 include="-I$pkgincludedir"
 
 # Remove some options that a client doesn't have to care about
 # FIXME until we have a --cxxflags, we need to remove -Xa
 #       and -xstrconst to make --cflags usable for Sun Forte C++
+# FIXME until we have a --cxxflags, we need to remove -AC99
+#       to make --cflags usable for HP C++ (aCC)
 for remove in DDBUG_OFF DSAFEMALLOC USAFEMALLOC DSAFE_MUTEX \
               DPEDANTIC_SAFEMALLOC DUNIV_MUST_NOT_INLINE DFORCE_INIT_OF_VARS \
               DEXTRA_DEBUG DHAVE_purify O 'O[0-9]' 'xO[0-9]' 'W[-A-Za-z]*' \
-              Xa xstrconst "xc99=none" \
+              'mtune=[-A-Za-z0-9]*' 'mcpu=[-A-Za-z0-9]*' 'march=[-A-Za-z0-9]*' \
+              Xa xstrconst "xc99=none" AC99 \
               unroll2 ip mp restrict
 do
   # The first option we might strip will always have a space before it because
