@@ -1,4 +1,5 @@
-/* Copyright (C) 2004 MySQL AB
+/* Copyright (c) 2004-2006 MySQL AB, 2009 Sun Microsystems, Inc.
+   Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "parse.h"
 #include "commands.h"
@@ -177,7 +178,7 @@ Command *parse_command(Instance_map *map, const char *text)
     get_word(&text, &option_len, NONSPACE);
     option= text;
     if ((tmp= strchr(text, '=')) != NULL)
-      option_len= tmp - text;
+      option_len= (uint) (tmp - text);
     text+= option_len;
 
     get_word(&text, &word_len);
