@@ -1,4 +1,5 @@
-/* Copyright (C) 2004-2006 MySQL AB
+/* Copyright (c) 2004-2006 MySQL AB, 2009 Sun Microsystems, Inc.
+   Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "protocol.h"
 
@@ -53,11 +54,11 @@ int net_send_ok(struct st_net *net, unsigned long connection_id,
   int2store(pos, 0);
   pos+= 2;
 
-  uint position= pos - buff.buffer; /* we might need it for message */
+  uint position= (uint) (pos - buff.buffer); /* we might need it for message */
 
   if (message != NULL)
   {
-    buff.reserve(position, 9 + strlen(message));
+    buff.reserve(position, 9 + (uint) strlen(message));
     store_to_protocol_packet(&buff, message, &position);
   }
 
@@ -82,7 +83,7 @@ int net_send_error(struct st_net *net, uint sql_errno)
   memcpy(pos, errno_to_sqlstate(sql_errno), SQLSTATE_LENGTH);
   pos+= SQLSTATE_LENGTH;
   pos= strmake(pos, err, MYSQL_ERRMSG_SIZE - 1) + 1;
-  return my_net_write(net, buff, pos - buff) || net_flush(net);
+  return my_net_write(net, buff, (uint) (pos - buff)) || net_flush(net);
 }
 
 
@@ -98,7 +99,7 @@ int net_send_error_323(struct st_net *net, uint sql_errno)
   int2store(pos, sql_errno);
   pos+= 2;
   pos= strmake(pos, err, MYSQL_ERRMSG_SIZE - 1) + 1;
-  return my_net_write(net, buff, pos - buff) || net_flush(net);
+  return my_net_write(net, buff, (uint) (pos - buff)) || net_flush(net);
 }
 
 char *net_store_length(char *pkg, uint length)
@@ -123,7 +124,7 @@ int store_to_protocol_packet(Buffer *buf, const char *string, uint *position,
   /* reserve max amount of bytes needed to store length */
   if (buf->reserve(*position, 9))
     goto err;
-  currpos= (net_store_length(buf->buffer + *position,
+  currpos= (uint) (net_store_length(buf->buffer + *position,
                              (ulonglong) string_len) - buf->buffer);
   if (buf->append(currpos, string, string_len))
     goto err;
@@ -139,7 +140,7 @@ int store_to_protocol_packet(Buffer *buf, const char *string, uint *position)
 {
   uint string_len;
 
-  string_len= strlen(string);
+  string_len= (uint) strlen(string);
   return store_to_protocol_packet(buf, string, position, string_len);
 }
 
