@@ -1,4 +1,5 @@
-/* Copyright (C) 2004-2006 MySQL AB
+/* Copyright (c) 2004-2006 MySQL AB, 2009 Sun Microsystems, Inc.
+   Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #if defined(__GNUC__) && defined(USE_PRAGMA_IMPLEMENTATION)
 #pragma implementation
@@ -55,7 +56,7 @@ int User::init(const char *line)
       goto err;
     password= name_end + 1;
   }
-  user_length= name_end - name_begin;
+  user_length= (uint) (name_end - name_begin);
   if (user_length > USERNAME_LENGTH)
     goto err;
 
@@ -145,8 +146,8 @@ int User_map::load(const char *password_file_name)
   while (fgets(line, sizeof(line), file))
   {
     /* skip comments and empty lines */
-    if (line[0] == '#' || line[0] == '\n' &&
-        (line[1] == '\0' || line[1] == '\r'))
+    if (line[0] == '#' || (line[0] == '\n' &&
+        (line[1] == '\0' || line[1] == '\r')))
       continue;
     if ((user= new User) == 0)
       goto done;
