@@ -62,7 +62,7 @@ public:
 
   void sort(CMP_FUNC cmp_func)
   {
-    qsort(array.buffer, array.elements, sizeof(Elem), (qsort_cmp)cmp_func);
+    my_qsort(array.buffer, array.elements, sizeof(Elem), (qsort_cmp)cmp_func);
   }
 };
 
