@@ -1,9 +1,9 @@
-/* Copyright (C) 2004-2005 MySQL AB
+/* Copyright (c) 2004-2008 MySQL AB, 2008, 2009 Sun Microsystems, Inc.
+   Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 
 #define MYSQL_LEX 1
@@ -36,17 +36,17 @@ static File_option triggers_file_parameters[]=
 {
   {
     {(char *) STRING_WITH_LEN("triggers") },
-    offsetof(class Table_triggers_list, definitions_list),
+    my_offsetof(class Table_triggers_list, definitions_list),
     FILE_OPTIONS_STRLIST
   },
   {
     {(char *) STRING_WITH_LEN("sql_modes") },
-    offsetof(class Table_triggers_list, definition_modes_list),
+    my_offsetof(class Table_triggers_list, definition_modes_list),
     FILE_OPTIONS_ULLLIST
   },
   {
     {(char *) STRING_WITH_LEN("definers") },
-    offsetof(class Table_triggers_list, definers_list),
+    my_offsetof(class Table_triggers_list, definers_list),
     FILE_OPTIONS_STRLIST
   },
   { { 0, 0 }, 0, FILE_OPTIONS_STRING }
@@ -55,7 +55,7 @@ static File_option triggers_file_parameters[]=
 File_option sql_modes_parameters=
 {
   {(char*) STRING_WITH_LEN("sql_modes") },
-  offsetof(class Table_triggers_list, definition_modes_list),
+  my_offsetof(class Table_triggers_list, definition_modes_list),
   FILE_OPTIONS_ULLLIST
 };
 
@@ -107,8 +107,6 @@ const LEX_STRING trg_event_type_names[]=
 };
 
 
-static TABLE_LIST *add_table_for_trigger(THD *thd, sp_name *trig);
-
 class Handle_old_incorrect_sql_modes_hook: public Unknown_key_hook
 {
 private:
@@ -156,13 +154,22 @@ private:
 */
 bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
 {
+  /*
+    FIXME: The code below takes too many different paths depending on the
+    'create' flag, so that the justification for a single function
+    'mysql_create_or_drop_trigger', compared to two separate functions
+    'mysql_create_trigger' and 'mysql_drop_trigger' is not apparent.
+    This is a good candidate for a minor refactoring.
+  */
   TABLE *table;
   bool result= TRUE;
-  LEX_STRING definer_user;
-  LEX_STRING definer_host;
+  String stmt_query;
 
   DBUG_ENTER("mysql_create_or_drop_trigger");
 
+  /* Charset of the buffer for statement must be system one. */
+  stmt_query.set_charset(system_charset_info);
+
   /*
     QQ: This function could be merged in mysql_alter_table() function
     But do we want this ?
@@ -173,16 +180,12 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
     need second part of condition below, since check_access() function also
     checks that db is specified.
   */
-  if (!thd->lex->spname->m_db.length || create && !tables->db_length)
+  if (!thd->lex->spname->m_db.length || (create && !tables->db_length))
   {
     my_error(ER_NO_DB_ERROR, MYF(0));
     DBUG_RETURN(TRUE);
   }
 
-  if (!create &&
-      !(tables= add_table_for_trigger(thd, thd->lex->spname)))
-    DBUG_RETURN(TRUE);
-
   /*
     We don't allow creating triggers on tables in the 'mysql' schema
   */
@@ -192,9 +195,6 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
     DBUG_RETURN(TRUE);
   }
 
-  /* We should have only one table in table list. */
-  DBUG_ASSERT(tables->next_global == 0);
-
   /*
     TODO: We should check if user has TRIGGER privilege for table here.
     Now we just require SUPER privilege for creating/dropping because
@@ -209,7 +209,7 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
     DROP for example) so we do the check for privileges. For now there is
     already a stronger test right above; but when this stronger test will
     be removed, the test below will hold. Because triggers have the same
-    nature as functions regarding binlogging: their body is implicitely
+    nature as functions regarding binlogging: their body is implicitly
     binlogged, so they share the same danger, so trust_function_creators
     applies to them too.
   */
@@ -220,31 +220,59 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
     DBUG_RETURN(TRUE);
   }
 
-  /* We do not allow creation of triggers on temporary tables. */
-  if (create && find_temporary_table(thd, tables->db, tables->table_name))
-  {
-    my_error(ER_TRG_ON_VIEW_OR_TEMP_TABLE, MYF(0), tables->alias);
-    DBUG_RETURN(TRUE);
-  }
-
   /*
     We don't want perform our operations while global read lock is held
-    so we have to wait until its end and then prevent it from occuring
+    so we have to wait until its end and then prevent it from occurring
     again until we are done. (Acquiring LOCK_open is not enough because
-    global read lock is held without helding LOCK_open).
+    global read lock is held without holding LOCK_open).
   */
   if (wait_if_global_read_lock(thd, 0, 1))
     DBUG_RETURN(TRUE);
 
   VOID(pthread_mutex_lock(&LOCK_open));
 
+  if (!create)
+  {
+    bool if_exists= thd->lex->drop_if_exists;
+
+    if (add_table_for_trigger(thd, thd->lex->spname, if_exists, & tables))
+      goto end;
+
+    if (!tables)
+    {
+      DBUG_ASSERT(if_exists);
+      /*
+        Since the trigger does not exist, there is no associated table,
+        and therefore :
+        - no TRIGGER privileges to check,
+        - no trigger to drop,
+        - no table to lock/modify,
+        so the drop statement is successful.
+      */
+      result= FALSE;
+      /* Still, we need to log the query ... */
+      stmt_query.append(thd->query, thd->query_length);
+      goto end;
+    }
+  }
+
+  /* We should have only one table in table list. */
+  DBUG_ASSERT(tables->next_global == 0);
+
+  /* We do not allow creation of triggers on temporary tables. */
+  if (create && find_temporary_table(thd, tables->db, tables->table_name))
+  {
+    my_error(ER_TRG_ON_VIEW_OR_TEMP_TABLE, MYF(0), tables->alias);
+    goto end;
+  }
+
   if (lock_table_names(thd, tables))
     goto end;
 
   /* We also don't allow creation of triggers on views. */
   tables->required_type= FRMTYPE_TABLE;
 
-  if (reopen_name_locked_table(thd, tables))
+  if (reopen_name_locked_table(thd, tables, TRUE))
   {
     unlock_table_name(thd, tables);
     goto end;
@@ -264,12 +292,10 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
   }
 
   result= (create ?
-           table->triggers->create_trigger(thd, tables, &definer_user, &definer_host):
-           table->triggers->drop_trigger(thd, tables));
+           table->triggers->create_trigger(thd, tables, &stmt_query):
+           table->triggers->drop_trigger(thd, tables, &stmt_query));
 
 end:
-  VOID(pthread_mutex_unlock(&LOCK_open));
-  start_waiting_global_read_lock(thd);
 
   if (!result)
   {
@@ -277,35 +303,19 @@ end:
     {
       thd->clear_error();
 
-      String log_query(thd->query, thd->query_length, system_charset_info);
-
-      if (create)
-      {
-        log_query.set((char *) 0, 0, system_charset_info); /* reset log_query */
-
-        log_query.append(STRING_WITH_LEN("CREATE "));
-
-        if (definer_user.str && definer_host.str)
-        {
-          /*
-            Append definer-clause if the trigger is SUID (a usual trigger in
-            new MySQL versions).
-          */
-
-          append_definer(thd, &log_query, &definer_user, &definer_host);
-        }
-
-        log_query.append(thd->lex->stmt_definition_begin);
-      }
-
       /* Such a statement can always go directly to binlog, no trans cache. */
-      Query_log_event qinfo(thd, log_query.ptr(), log_query.length(), 0, FALSE);
+      Query_log_event qinfo(thd, stmt_query.ptr(), stmt_query.length(), 0,
+                            FALSE, THD::NOT_KILLED);
       mysql_bin_log.write(&qinfo);
     }
-
-    send_ok(thd);
   }
 
+  VOID(pthread_mutex_unlock(&LOCK_open));
+  start_waiting_global_read_lock(thd);
+
+  if (!result)
+    send_ok(thd);
+
   DBUG_RETURN(result);
 }
 
@@ -319,22 +329,8 @@ end:
                      LEX)
       tables       - table list containing one open table for which the
                      trigger is created.
-      definer_user - [out] after a call it points to 0-terminated string or
-                     contains the NULL-string:
-                       - 0-terminated is returned if the trigger is SUID. The
-                         string contains user name part of the actual trigger
-                         definer.
-                       - NULL-string is returned if the trigger is non-SUID.
-                     Anyway, the caller is responsible to provide memory for
-                     storing LEX_STRING object.
-      definer_host - [out] after a call it points to 0-terminated string or
-                     contains the NULL-string:
-                       - 0-terminated string is returned if the trigger is
-                         SUID. The string contains host name part of the
-                         actual trigger definer.
-                       - NULL-string is returned if the trigger is non-SUID.
-                     Anyway, the caller is responsible to provide memory for
-                     storing LEX_STRING object.
+      stmt_query   - [OUT] after successful return, this string contains
+                     well-formed statement for creation this trigger.
 
   NOTE
     - Assumes that trigger name is fully qualified.
@@ -349,15 +345,16 @@ end:
     True  - error
 */
 bool Table_triggers_list::create_trigger(THD *thd, TABLE_LIST *tables,
-                                         LEX_STRING *definer_user,
-                                         LEX_STRING *definer_host)
+                                         String *stmt_query)
 {
   LEX *lex= thd->lex;
   TABLE *table= tables->table;
   char dir_buff[FN_REFLEN], file_buff[FN_REFLEN], trigname_buff[FN_REFLEN],
        trigname_path[FN_REFLEN];
   LEX_STRING dir, file, trigname_file;
-  LEX_STRING *trg_def, *name;
+  LEX_STRING *trg_def;
+  LEX_STRING definer_user;
+  LEX_STRING definer_host;
   ulonglong *trg_sql_mode;
   char trg_definer_holder[USER_HOST_BUFF_SIZE];
   LEX_STRING *trg_definer;
@@ -375,7 +372,9 @@ bool Table_triggers_list::create_trigger(THD *thd, TABLE_LIST *tables,
   /* We don't allow creation of several triggers of the same type yet */
   if (bodies[lex->trg_chistics.event][lex->trg_chistics.action_time])
   {
-    my_message(ER_TRG_ALREADY_EXISTS, ER(ER_TRG_ALREADY_EXISTS), MYF(0));
+    my_error(ER_NOT_SUPPORTED_YET, MYF(0),
+             "multiple triggers with the same action time"
+             " and event for one table");
     return 1;
   }
 
@@ -461,12 +460,12 @@ bool Table_triggers_list::create_trigger(THD *thd, TABLE_LIST *tables,
   strxnmov(dir_buff, FN_REFLEN, mysql_data_home, "/", tables->db, "/", NullS);
   dir.length= unpack_filename(dir_buff, dir_buff);
   dir.str= dir_buff;
-  file.length=  strxnmov(file_buff, FN_REFLEN, tables->table_name,
-                         triggers_file_ext, NullS) - file_buff;
+  file.length= (uint) (strxnmov(file_buff, FN_REFLEN, tables->table_name,
+                         triggers_file_ext, NullS) - file_buff);
   file.str= file_buff;
-  trigname_file.length= strxnmov(trigname_buff, FN_REFLEN,
+  trigname_file.length= (uint) (strxnmov(trigname_buff, FN_REFLEN,
                                  lex->spname->m_name.str,
-                                 trigname_file_ext, NullS) - trigname_buff;
+                                 trigname_file_ext, NullS) - trigname_buff);
   trigname_file.str= trigname_buff;
   strxnmov(trigname_path, FN_REFLEN, dir_buff, trigname_buff, NullS);
 
@@ -481,7 +480,7 @@ bool Table_triggers_list::create_trigger(THD *thd, TABLE_LIST *tables,
   trigname.trigger_table.length= tables->table_name_length;
 
   if (sql_create_definition_file(&dir, &trigname_file, &trigname_file_type,
-                                 (gptr)&trigname, trigname_file_parameters, 0))
+                                 (gptr)&trigname, trigname_file_parameters))
     return 1;
 
   /*
@@ -503,8 +502,6 @@ bool Table_triggers_list::create_trigger(THD *thd, TABLE_LIST *tables,
       definers_list.push_back(trg_definer, &table->mem_root))
     goto err_with_cleanup;
 
-  trg_def->str= thd->query;
-  trg_def->length= thd->query_length;
   *trg_sql_mode= thd->variables.sql_mode;
 
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
@@ -524,29 +521,56 @@ bool Table_triggers_list::create_trigger(THD *thd, TABLE_LIST *tables,
   {
     /* SUID trigger. */
 
-    *definer_user= lex->definer->user;
-    *definer_host= lex->definer->host;
+    definer_user= lex->definer->user;
+    definer_host= lex->definer->host;
 
     trg_definer->str= trg_definer_holder;
-    trg_definer->length= strxmov(trg_definer->str, definer_user->str, "@",
-                                 definer_host->str, NullS) - trg_definer->str;
+    trg_definer->length= (uint) (strxmov(trg_definer->str, definer_user.str, "@",
+                                 definer_host.str, NullS) - trg_definer->str);
   }
   else
   {
     /* non-SUID trigger. */
 
-    definer_user->str= 0;
-    definer_user->length= 0;
+    definer_user.str= 0;
+    definer_user.length= 0;
 
-    definer_host->str= 0;
-    definer_host->length= 0;
+    definer_host.str= 0;
+    definer_host.length= 0;
 
     trg_definer->str= (char*) "";
     trg_definer->length= 0;
   }
 
+  /*
+    Create well-formed trigger definition query. Original query is not
+    appropriated, because definer-clause can be not truncated.
+  */
+
+  stmt_query->append(STRING_WITH_LEN("CREATE "));
+
+  if (trg_definer)
+  {
+    /*
+      Append definer-clause if the trigger is SUID (a usual trigger in
+      new MySQL versions).
+    */
+
+    append_definer(thd, stmt_query, &definer_user, &definer_host);
+  }
+
+  stmt_query->append(thd->lex->stmt_definition_begin,
+                     (uint) ((char *) thd->lex->sphead->m_body_begin -
+                     thd->lex->stmt_definition_begin +
+                     thd->lex->sphead->m_body.length));
+
+  trg_def->str= stmt_query->c_ptr();
+  trg_def->length= stmt_query->length();
+
+  /* Create trigger definition file. */
+
   if (!sql_create_definition_file(&dir, &file, &triggers_file_type,
-                                  (gptr)this, triggers_file_parameters, 0))
+                                  (gptr)this, triggers_file_parameters))
     return 0;
 
 err_with_cleanup:
@@ -628,12 +652,12 @@ static bool save_trigger_file(Table_triggers_list *triggers, const char *db,
   strxnmov(dir_buff, FN_REFLEN, mysql_data_home, "/", db, "/", NullS);
   dir.length= unpack_filename(dir_buff, dir_buff);
   dir.str= dir_buff;
-  file.length=  strxnmov(file_buff, FN_REFLEN, table_name, triggers_file_ext,
-                         NullS) - file_buff;
+  file.length= (uint) (strxnmov(file_buff, FN_REFLEN, table_name, triggers_file_ext,
+                         NullS) - file_buff);
   file.str= file_buff;
 
   return sql_create_definition_file(&dir, &file, &triggers_file_type,
-                                    (gptr)triggers, triggers_file_parameters, 0);
+                                    (gptr)triggers, triggers_file_parameters);
 }
 
 
@@ -642,15 +666,19 @@ static bool save_trigger_file(Table_triggers_list *triggers, const char *db,
 
   SYNOPSIS
     drop_trigger()
-      thd    - current thread context (including trigger definition in LEX)
-      tables - table list containing one open table for which trigger is
-               dropped.
+      thd         - current thread context
+                    (including trigger definition in LEX)
+      tables      - table list containing one open table for which trigger
+                    is dropped.
+      stmt_query  - [OUT] after successful return, this string contains
+                    well-formed statement for creation this trigger.
 
   RETURN VALUE
     False - success
     True  - error
 */
-bool Table_triggers_list::drop_trigger(THD *thd, TABLE_LIST *tables)
+bool Table_triggers_list::drop_trigger(THD *thd, TABLE_LIST *tables,
+                                       String *stmt_query)
 {
   LEX *lex= thd->lex;
   LEX_STRING *name;
@@ -660,6 +688,8 @@ bool Table_triggers_list::drop_trigger(THD *thd, TABLE_LIST *tables)
   List_iterator<LEX_STRING>      it_definer(definers_list);
   char path[FN_REFLEN];
 
+  stmt_query->append(thd->query, thd->query_length);
+
   while ((name= it_name++))
   {
     it_def++;
@@ -745,7 +775,8 @@ bool Table_triggers_list::prepare_record1_accessors(TABLE *table)
       QQ: it is supposed that it is ok to use this function for field
       cloning...
     */
-    if (!(*old_fld= (*fld)->new_field(&table->mem_root, table)))
+    if (!(*old_fld= (*fld)->new_field(&table->mem_root, table,
+                                      table == (*fld)->table)))
       return 1;
     (*old_fld)->move_field((my_ptrdiff_t)(table->record[1] -
                                           table->record[0]));
@@ -766,8 +797,8 @@ bool Table_triggers_list::prepare_record1_accessors(TABLE *table)
 
 void Table_triggers_list::set_table(TABLE *new_table)
 {
-  table= new_table;
-  for (Field **field= table->triggers->record1_field ; *field ; field++)
+  trigger_table= new_table;
+  for (Field **field= new_table->triggers->record1_field ; *field ; field++)
   {
     (*field)->table= (*field)->orig_table= new_table;
     (*field)->table_name= &new_table->alias;
@@ -844,7 +875,7 @@ bool Table_triggers_list::check_n_load(THD *thd, const char *db,
         DBUG_RETURN(1);
 
       List_iterator_fast<LEX_STRING> it(triggers->definitions_list);
-      LEX_STRING *trg_create_str, *trg_name_str;
+      LEX_STRING *trg_create_str;
       ulonglong *trg_sql_mode;
 
       if (triggers->definition_modes_list.is_empty() &&
@@ -912,17 +943,6 @@ bool Table_triggers_list::check_n_load(THD *thd, const char *db,
 
       table->triggers= triggers;
 
-      /*
-        Construct key that will represent triggers for this table in the set
-        of routines used by statement.
-      */
-      triggers->sroutines_key.length= 1+strlen(db)+1+strlen(table_name)+1;
-      if (!(triggers->sroutines_key.str=
-              alloc_root(&table->mem_root, triggers->sroutines_key.length)))
-        DBUG_RETURN(1);
-      triggers->sroutines_key.str[0]= TYPE_ENUM_TRIGGER;
-      strxmov(triggers->sroutines_key.str+1, db, ".", table_name, NullS);
-
       /*
         TODO: This could be avoided if there is no triggers
               for UPDATE and DELETE.
@@ -941,27 +961,39 @@ bool Table_triggers_list::check_n_load(THD *thd, const char *db,
 
       save_db.str= thd->db;
       save_db.length= thd->db_length;
-      thd->reset_db((char*) db, strlen(db));
+      thd->reset_db((char*) db, (uint) strlen(db));
       while ((trg_create_str= it++))
       {
         trg_sql_mode= itm++;
         LEX_STRING *trg_definer= it_definer++;
 
         thd->variables.sql_mode= (ulong)*trg_sql_mode;
-        lex_start(thd, (uchar*)trg_create_str->str, trg_create_str->length);
 
-	thd->spcont= 0;
-        if (MYSQLparse((void *)thd) || thd->is_fatal_error)
+        Parser_state parser_state(thd, trg_create_str->str,
+                                  trg_create_str->length);
+        thd->m_parser_state= &parser_state;
+        lex_start(thd);
+        thd->spcont= NULL;
+        int err= MYSQLparse((void *)thd);
+        thd->m_parser_state= NULL;
+
+        if (err || thd->is_fatal_error)
         {
-          /*
-            Free lex associated resources.
-            QQ: Do we really need all this stuff here ?
-          */
-          delete lex.sphead;
+          /* Currently sphead is always deleted in case of a parse error */
+          DBUG_ASSERT(lex.sphead == 0);
           goto err_with_lex_cleanup;
         }
+        /*
+          Not strictly necessary to invoke this method here, since we know
+          that we've parsed CREATE TRIGGER and not an
+          UPDATE/DELETE/INSERT/REPLACE/LOAD/CREATE TABLE, but we try to
+          maintain the invariant that this method is called for each
+          distinct statement, in case its logic is extended with other
+          types of analyses in future.
+        */
+        lex.set_trg_event_type_for_tables();
 
-        lex.sphead->set_info(0, 0, &lex.sp_chistics, *trg_sql_mode);
+        lex.sphead->set_info(0, 0, &lex.sp_chistics, (ulong) *trg_sql_mode);
 
         triggers->bodies[lex.trg_chistics.event]
                              [lex.trg_chistics.action_time]= lex.sphead;
@@ -1022,8 +1054,15 @@ bool Table_triggers_list::check_n_load(THD *thd, const char *db,
         }
 
         /*
-          Let us bind Item_trigger_field objects representing access to fields
-          in old/new versions of row in trigger to Field objects in table being
+          Gather all Item_trigger_field objects representing access to fields
+          in old/new versions of row in trigger into lists containing all such
+          objects for the triggers with same action and timing.
+        */
+        triggers->trigger_fields[lex.trg_chistics.event]
+                                [lex.trg_chistics.action_time]=
+          (Item_trigger_field *)(lex.trg_table_fields.first);
+        /*
+          Also let us bind these objects to Field objects in table being
           opened.
 
           We ignore errors here, because if even something is wrong we still
@@ -1115,8 +1154,8 @@ bool Table_triggers_list::get_trigger_info(THD *thd, trg_event_type event,
     }
     else
     {
-      definer->length= strxmov(definer->str, body->m_definer_user.str, "@",
-                               body->m_definer_host.str, NullS) - definer->str;
+      definer->length= (uint) (strxmov(definer->str, body->m_definer_user.str, "@",
+                               body->m_definer_host.str, NullS) - definer->str);
     }
 
     DBUG_RETURN(0);
@@ -1133,13 +1172,17 @@ bool Table_triggers_list::get_trigger_info(THD *thd, trg_event_type event,
     mysql_table_for_trigger()
       thd    - current thread context
       trig   - identifier for trigger
+      if_exists - treat a not existing trigger as a warning if TRUE
+      table - pointer to TABLE_LIST object for the table trigger (output)
 
   RETURN VALUE
-    0 - error
-    # - pointer to TABLE_LIST object for the table
+    0 Success
+    1 Error
 */
 
-static TABLE_LIST *add_table_for_trigger(THD *thd, sp_name *trig)
+int
+add_table_for_trigger(THD *thd, sp_name *trig, bool if_exists,
+                      TABLE_LIST **table)
 {
   LEX *lex= thd->lex;
   char path_buff[FN_REFLEN];
@@ -1150,6 +1193,7 @@ static TABLE_LIST *add_table_for_trigger(THD *thd, sp_name *trig)
                                           path_buff, &trigname.trigger_table);
   
   DBUG_ENTER("add_table_for_trigger");
+  DBUG_ASSERT(table != NULL);
 
   strxnmov(path_buff, FN_REFLEN, mysql_data_home, "/", trig->m_db.str, "/",
            trig->m_name.str, trigname_file_ext, NullS);
@@ -1158,30 +1202,45 @@ static TABLE_LIST *add_table_for_trigger(THD *thd, sp_name *trig)
 
   if (access(path_buff, F_OK))
   {
+    if (if_exists)
+    {
+      push_warning_printf(thd,
+                         MYSQL_ERROR::WARN_LEVEL_NOTE,
+                         ER_TRG_DOES_NOT_EXIST,
+                         ER(ER_TRG_DOES_NOT_EXIST));
+      *table= NULL;
+      DBUG_RETURN(0);
+    }
+
     my_error(ER_TRG_DOES_NOT_EXIST, MYF(0));
-    DBUG_RETURN(0);
+    DBUG_RETURN(1);
   }
 
   if (!(parser= sql_parse_prepare(&path, thd->mem_root, 1)))
-    DBUG_RETURN(0);
+    DBUG_RETURN(1);
 
   if (!is_equal(&trigname_file_type, parser->type()))
   {
     my_error(ER_WRONG_OBJECT, MYF(0), trig->m_name.str, trigname_file_ext+1,
              "TRIGGERNAME");
-    DBUG_RETURN(0);
+    DBUG_RETURN(1);
   }
 
   if (parser->parse((gptr)&trigname, thd->mem_root,
                     trigname_file_parameters, 1,
                     &trigger_table_hook))
-    DBUG_RETURN(0);
+    DBUG_RETURN(1);
 
   /* We need to reset statement table list to be PS/SP friendly. */
   lex->query_tables= 0;
   lex->query_tables_last= &lex->query_tables;
-  DBUG_RETURN(sp_add_to_query_tables(thd, lex, trig->m_db.str,
-                                     trigname.trigger_table.str, TL_IGNORE));
+  *table= sp_add_to_query_tables(thd, lex, trig->m_db.str,
+                                 trigname.trigger_table.str, TL_IGNORE);
+
+  if (! *table)
+    DBUG_RETURN(1);
+
+  DBUG_RETURN(0);
 }
 
 
@@ -1275,7 +1334,6 @@ Table_triggers_list::change_table_name_in_triggers(THD *thd,
 {
   char path_buff[FN_REFLEN];
   LEX_STRING *def, *on_table_name, new_def;
-  ulonglong *sql_mode;
   ulong save_sql_mode= thd->variables.sql_mode;
   List_iterator_fast<LEX_STRING> it_def(definitions_list);
   List_iterator_fast<LEX_STRING> it_on_table_name(on_table_names_list);
@@ -1289,11 +1347,11 @@ Table_triggers_list::change_table_name_in_triggers(THD *thd,
   while ((def= it_def++))
   {
     on_table_name= it_on_table_name++;
-    thd->variables.sql_mode= *(it_mode++);
+    thd->variables.sql_mode= (ulong) *(it_mode++);
 
     /* Construct CREATE TRIGGER statement with new table name. */
     buff.length(0);
-    before_on_len= on_table_name->str - def->str;
+    before_on_len= (uint) (on_table_name->str - def->str);
     buff.append(def->str, before_on_len);
     buff.append(STRING_WITH_LEN("ON "));
     append_identifier(thd, &buff, new_table_name->str, new_table_name->length);
@@ -1305,7 +1363,8 @@ Table_triggers_list::change_table_name_in_triggers(THD *thd,
       It is OK to allocate some memory on table's MEM_ROOT since this
       table instance will be thrown out at the end of rename anyway.
     */
-    new_def.str= memdup_root(&table->mem_root, buff.ptr(), buff.length());
+    new_def.str= memdup_root(&trigger_table->mem_root, buff.ptr(),
+                             buff.length());
     new_def.length= buff.length();
     on_table_name->str= new_def.str + before_on_len;
     on_table_name->length= on_q_table_name_len;
@@ -1362,14 +1421,14 @@ Table_triggers_list::change_table_name_in_trignames(const char *db_name,
 
   while ((trigger= it_name++) != stopper)
   {
-    trigname_file.length= strxnmov(trigname_buff, FN_REFLEN, trigger->str,
-                                   trigname_file_ext, NullS) - trigname_buff;
+    trigname_file.length= (uint) (strxnmov(trigname_buff, FN_REFLEN, trigger->str,
+                                   trigname_file_ext, NullS) - trigname_buff);
     trigname_file.str= trigname_buff;
 
     trigname.trigger_table= *new_table_name;
 
     if (sql_create_definition_file(&dir, &trigname_file, &trigname_file_type,
-        (gptr)&trigname, trigname_file_parameters, 0))
+        (gptr)&trigname, trigname_file_parameters))
       return trigger;
   }
 
@@ -1424,8 +1483,8 @@ bool Table_triggers_list::change_table_name(THD *thd, const char *db,
   }
   if (table.triggers)
   {
-    LEX_STRING_WITH_INIT old_table_name(old_table, strlen(old_table));
-    LEX_STRING_WITH_INIT new_table_name(new_table, strlen(new_table));
+    LEX_STRING_WITH_INIT old_table_name(old_table, (uint) strlen(old_table));
+    LEX_STRING_WITH_INIT new_table_name(new_table, (uint) strlen(new_table));
     /*
       Since triggers should be in the same schema as their subject tables
       moving table with them between two schemas raises too many questions.
@@ -1485,54 +1544,96 @@ bool Table_triggers_list::process_triggers(THD *thd, trg_event_type event,
     if (old_row_is_record1)
     {
       old_field= record1_field;
-      new_field= table->field;
+      new_field= trigger_table->field;
     }
     else
     {
       new_field= record1_field;
-      old_field= table->field;
+      old_field= trigger_table->field;
     }
-
-#ifndef NO_EMBEDDED_ACCESS_CHECKS
-    Security_context *save_ctx;
-
-    if (sp_change_security_context(thd, sp_trigger, &save_ctx))
-      return TRUE;
-
     /*
-      NOTE: TRIGGER_ACL should be used below.
+      This trigger must have been processed by the pre-locking
+      algorithm.
     */
-
-    if (check_global_access(thd, SUPER_ACL))
-    {
-      sp_restore_security_context(thd, save_ctx);
-      return TRUE;
-    }
-
-    /*
-      Fetch information about table-level privileges to GRANT_INFO structure for
-      subject table. Check of privileges that will use it and information about
-      column-level privileges will happen in Item_trigger_field::fix_fields().
-    */
-
-    fill_effective_table_privileges(thd,
-                                    &subject_table_grants[event][time_type],
-                                    table->s->db, table->s->table_name);
-#endif // NO_EMBEDDED_ACCESS_CHECKS
+    DBUG_ASSERT(trigger_table->pos_in_table_list->trg_event_map &
+                static_cast<uint>(1 << static_cast<int>(event)));
 
     thd->reset_sub_statement_state(&statement_state, SUB_STMT_TRIGGER);
-    err_status= sp_trigger->execute_function(thd, 0, 0, 0);
+    err_status= sp_trigger->execute_trigger
+      (thd, trigger_table->s->db, trigger_table->s->table_name,
+       &subject_table_grants[event][time_type]);
     thd->restore_sub_statement_state(&statement_state);
-
-#ifndef NO_EMBEDDED_ACCESS_CHECKS
-    sp_restore_security_context(thd, save_ctx);
-#endif // NO_EMBEDDED_ACCESS_CHECKS
   }
 
   return err_status;
 }
 
 
+/*
+  Mark fields of subject table which we read/set in its triggers as such.
+
+  SYNOPSIS
+    mark_fields_used()
+      thd    Current thread context
+      event  Type of event triggers for which we are going to ins
+
+  DESCRIPTION
+    This method marks fields of subject table which are read/set in its
+    triggers as such (by setting Field::query_id equal to THD::query_id)
+    and thus informs handler that values for these fields should be
+    retrieved/stored during execution of statement.
+*/
+
+void Table_triggers_list::mark_fields_used(THD *thd, trg_event_type event)
+{
+  int action_time;
+  Item_trigger_field *trg_field;
+
+  for (action_time= 0; action_time < (int)TRG_ACTION_MAX; action_time++)
+  {
+    for (trg_field= trigger_fields[event][action_time]; trg_field;
+         trg_field= trg_field->next_trg_field)
+    {
+      /* We cannot mark fields which does not present in table. */
+      if (trg_field->field_idx != (uint)-1)
+        trigger_table->field[trg_field->field_idx]->query_id = thd->query_id;
+    }
+  }
+}
+
+
+/*
+  Check if field of subject table can be changed in before update trigger.
+
+  SYNOPSIS
+    is_updated_in_before_update_triggers()
+      field  Field object for field to be checked
+
+  NOTE
+    Field passed to this function should be bound to the same
+    TABLE object as Table_triggers_list.
+
+  RETURN VALUE
+    TRUE   Field is changed
+    FALSE  Otherwise
+*/
+
+bool Table_triggers_list::is_updated_in_before_update_triggers(Field *fld)
+{
+  Item_trigger_field *trg_fld;
+  for (trg_fld= trigger_fields[TRG_EVENT_UPDATE][TRG_ACTION_BEFORE];
+       trg_fld != 0;
+       trg_fld= trg_fld->next_trg_field)
+  {
+    if (trg_fld->get_settable_routine_parameter() &&
+        trg_fld->field_idx != (uint)-1 &&
+        trigger_table->field[trg_fld->field_idx]->eq(fld))
+      return TRUE;
+  }
+  return FALSE;
+}
+
+
 /*
   Trigger BUG#14090 compatibility hook
 
