@@ -1,4 +1,5 @@
-/* Copyright (C) 2000 MySQL AB
+/* Copyright (c) 2002-2007 MySQL AB, 2009 Sun Microsystems, Inc.
+   Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <my_global.h>
 #include "m_ctype.h"
@@ -37,7 +38,7 @@ uint my_caseup_str_mb(CHARSET_INFO * cs, char *str)
       str++;
     }
   }
-  return str - str_orig;
+  return (uint) (str - str_orig);
 }
 
 uint my_casedn_str_mb(CHARSET_INFO * cs, char *str)
@@ -57,7 +58,7 @@ uint my_casedn_str_mb(CHARSET_INFO * cs, char *str)
       str++;
     }
   }
-  return str - str_orig;
+  return (uint) (str - str_orig);
 }
 
 uint my_caseup_mb(CHARSET_INFO * cs, char *src, uint srclen,
@@ -185,7 +186,7 @@ int my_wildcmp_mb(CHARSET_INFO *cs,
     {						/* Found w_many */
       uchar cmp;
       const char* mb = wildstr;
-      int mblen=0;
+      int mb_len=0;
       
       wildstr++;
       /* Remove any '%' and '_' from the wild search string */
@@ -211,7 +212,7 @@ int my_wildcmp_mb(CHARSET_INFO *cs,
 	cmp= *++wildstr;
 	
       mb=wildstr;
-      mblen= my_ismbchar(cs, wildstr, wildend);
+      mb_len= my_ismbchar(cs, wildstr, wildend);
       INC_PTR(cs,wildstr,wildend);		/* This is compared trough cmp */
       cmp=likeconv(cs,cmp);   
       do
@@ -220,11 +221,11 @@ int my_wildcmp_mb(CHARSET_INFO *cs,
         {
           if (str >= str_end)
             return -1;
-          if (mblen)
+          if (mb_len)
           {
-            if (str+mblen <= str_end && memcmp(str, mb, mblen) == 0)
+            if (str+mb_len <= str_end && memcmp(str, mb, mb_len) == 0)
             {
-              str += mblen;
+              str += mb_len;
               break;
             }
           }
@@ -256,8 +257,8 @@ uint my_numchars_mb(CHARSET_INFO *cs __attribute__((unused)),
   register uint32 count=0;
   while (pos < end) 
   {
-    uint mblen;
-    pos+= (mblen= my_ismbchar(cs,pos,end)) ? mblen : 1;
+    uint mb_len;
+    pos+= (mb_len= my_ismbchar(cs,pos,end)) ? mb_len : 1;
     count++;
   }
   return count;
@@ -271,8 +272,8 @@ uint my_charpos_mb(CHARSET_INFO *cs __attribute__((unused)),
   
   while (length && pos < end)
   {
-    uint mblen;
-    pos+= (mblen= my_ismbchar(cs, pos, end)) ? mblen : 1;
+    uint mb_len;
+    pos+= (mb_len= my_ismbchar(cs, pos, end)) ? mb_len : 1;
     length--;
   }
   return (uint) (length ? end+2-start : pos-start);
@@ -287,14 +288,14 @@ uint my_well_formed_len_mb(CHARSET_INFO *cs, const char *b, const char *e,
   while (pos)
   {
     my_wc_t wc;
-    int mblen;
+    int mb_len;
 
-    if ((mblen= cs->cset->mb_wc(cs, &wc, (uchar*) b, (uchar*) e)) <= 0)
+    if ((mb_len= cs->cset->mb_wc(cs, &wc, (uchar*) b, (uchar*) e)) <= 0)
     {
       *error= b < e ? 1 : 0;
       break;
     }
-    b+= mblen;
+    b+= mb_len;
     pos--;
   }
   return (uint) (b - b_start);
@@ -318,7 +319,7 @@ uint my_instr_mb(CHARSET_INFO *cs,
       {
         match->beg= 0;
         match->end= 0;
-        match->mblen= 0;
+        match->mb_len= 0;
       }
       return 1;		/* Empty string is always found */
     }
@@ -328,7 +329,7 @@ uint my_instr_mb(CHARSET_INFO *cs,
     
     while (b < end)
     {
-      int mblen;
+      int mb_len;
       
       if (!cs->coll->strnncoll(cs, (unsigned char*) b,   s_length, 
       				   (unsigned char*) s, s_length, 0))
@@ -337,19 +338,19 @@ uint my_instr_mb(CHARSET_INFO *cs,
         {
           match[0].beg= 0;
           match[0].end= (uint) (b-b0);
-          match[0].mblen= res;
+          match[0].mb_len= res;
           if (nmatch > 1)
           {
             match[1].beg= match[0].end;
             match[1].end= match[0].end+s_length;
-            match[1].mblen= 0;	/* Not computed */
+            match[1].mb_len= 0;	/* Not computed */
           }
         }
         return 2;
       }
-      mblen= (mblen= my_ismbchar(cs, b, end)) ? mblen : 1;
-      b+= mblen;
-      b_length-= mblen;
+      mb_len= (mb_len= my_ismbchar(cs, b, end)) ? mb_len : 1;
+      b+= mb_len;
+      b_length-= mb_len;
       res++;
     }
   }
@@ -467,6 +468,13 @@ static void my_hash_sort_mb_bin(CHARSET_INFO *cs __attribute__((unused)),
   
   key+= len;
   
+  /*
+     Remove trailing spaces. We have to do this to be able to compare
+    'A ' and 'A' as identical
+  */
+  while (key > pos && key[-1] == ' ')
+    key--;
+  
   for (; pos < (uchar*) key ; pos++)
   {
     nr1[0]^=(ulong) ((((uint) nr1[0] & 63)+nr2[0]) * 
@@ -550,12 +558,14 @@ my_bool my_like_range_mb(CHARSET_INFO *cs,
 			 char *min_str,char *max_str,
 			 uint *min_length,uint *max_length)
 {
-  uint mblen;
+  uint mb_len;
   const char *end= ptr + ptr_length;
   char *min_org= min_str;
   char *min_end= min_str + res_length;
   char *max_end= max_str + res_length;
   uint maxcharlen= res_length / cs->mbmaxlen;
+  const char *contraction_flags= cs->contractions ? 
+              ((const char*) cs->contractions) + 0x40*0x40 : NULL;
 
   for (; ptr != end && min_str != min_end && maxcharlen ; maxcharlen--)
   {
@@ -564,6 +574,7 @@ my_bool my_like_range_mb(CHARSET_INFO *cs,
       ptr++;                                    /* Skip escape */
     else if (*ptr == w_one || *ptr == w_many)   /* '_' and '%' in SQL */
     {      
+fill_max_and_min:
       /*
         Calculate length of keys:
         'a\0\0... is the smallest possible string when we have space expand
@@ -587,16 +598,82 @@ my_bool my_like_range_mb(CHARSET_INFO *cs,
       pad_max_char(cs, max_str, max_end);
       return 0;
     }
-    if ((mblen= my_ismbchar(cs, ptr, end)) > 1)
+    if ((mb_len= my_ismbchar(cs, ptr, end)) > 1)
     {
-      if (ptr+mblen > end || min_str+mblen > min_end)
+      if (ptr+mb_len > end || min_str+mb_len > min_end)
         break;
-      while (mblen--)
+      while (mb_len--)
        *min_str++= *max_str++= *ptr++;
     }
     else
-       *min_str++= *max_str++= *ptr++;    
+    {
+      /*
+        Special case for collations with contractions.
+        For example, in Chezh, 'ch' is a separate letter
+        which is sorted between 'h' and 'i'.
+        If the pattern 'abc%', 'c' at the end can mean:
+        - letter 'c' itself,
+        - beginning of the contraction 'ch'.
 
+        If we simply return this LIKE range:
+
+         'abc\min\min\min' and 'abc\max\max\max'
+
+        then this query: SELECT * FROM t1 WHERE a LIKE 'abc%'
+        will only find values starting from 'abc[^h]',
+        but won't find values starting from 'abch'.
+
+        We must ignore contraction heads followed by w_one or w_many.
+        ('Contraction head' means any letter which can be the first
+        letter in a contraction)
+
+        For example, for Czech 'abc%', we will return LIKE range,
+        which is equal to LIKE range for 'ab%':
+
+        'ab\min\min\min\min' and 'ab\max\max\max\max'.
+
+      */
+      if (contraction_flags && ptr + 1 < end &&
+          contraction_flags[(uchar) *ptr])
+      {
+        /* Ptr[0] is a contraction head. */
+        
+        if (ptr[1] == w_one || ptr[1] == w_many)
+        {
+          /* Contraction head followed by a wildcard, quit. */
+          goto fill_max_and_min;
+        }
+        
+        /*
+          Some letters can be both contraction heads and contraction tails.
+          For example, in Danish 'aa' is a separate single letter which
+          is sorted after 'z'. So 'a' can be both head and tail.
+          
+          If ptr[0]+ptr[1] is a contraction,
+          then put both letters together.
+          
+          If ptr[1] can be a contraction part, but ptr[0]+ptr[1]
+          is not a contraction, then we put only ptr[0],
+          and continue with ptr[1] on the next loop.
+        */
+        if (contraction_flags[(uchar) ptr[1]] &&
+            cs->contractions[(*ptr-0x40)*0x40 + ptr[1] - 0x40])
+        {
+          /* Contraction found */
+          if (maxcharlen == 1 || min_str + 1 >= min_end)
+          {
+            /* Both contraction parts don't fit, quit */
+            goto fill_max_and_min;
+          }
+          
+          /* Put contraction head */
+          *min_str++= *max_str++= *ptr++;
+          maxcharlen--;
+        }
+      }
+      /* Put contraction tail, or a single character */
+      *min_str++= *max_str++= *ptr++;    
+    }
   }
 
   *min_length= *max_length = (uint) (min_str - min_org);
@@ -649,7 +726,7 @@ static int my_wildcmp_mb_bin(CHARSET_INFO *cs,
     {						/* Found w_many */
       uchar cmp;
       const char* mb = wildstr;
-      int mblen=0;
+      int mb_len=0;
       
       wildstr++;
       /* Remove any '%' and '_' from the wild search string */
@@ -675,7 +752,7 @@ static int my_wildcmp_mb_bin(CHARSET_INFO *cs,
 	cmp= *++wildstr;
 	
       mb=wildstr;
-      mblen= my_ismbchar(cs, wildstr, wildend);
+      mb_len= my_ismbchar(cs, wildstr, wildend);
       INC_PTR(cs,wildstr,wildend);		/* This is compared trough cmp */
       do
       {
@@ -683,11 +760,11 @@ static int my_wildcmp_mb_bin(CHARSET_INFO *cs,
         {
           if (str >= str_end)
             return -1;
-          if (mblen)
+          if (mb_len)
           {
-            if (str+mblen <= str_end && memcmp(str, mb, mblen) == 0)
+            if (str+mb_len <= str_end && memcmp(str, mb, mb_len) == 0)
             {
-              str += mblen;
+              str += mb_len;
               break;
             }
           }
@@ -926,15 +1003,15 @@ uint my_numcells_mb(CHARSET_INFO *cs, const char *b, const char *e)
   
   while (b < e)
   {
-    int mblen;
+    int mb_len;
     uint pg;
-    if ((mblen= cs->cset->mb_wc(cs, &wc, (uchar*) b, (uchar*) e)) <= 0)
+    if ((mb_len= cs->cset->mb_wc(cs, &wc, (uchar*) b, (uchar*) e)) <= 0)
     {
-      mblen= 1; /* Let's think a wrong sequence takes 1 dysplay cell */
+      mb_len= 1; /* Let's think a wrong sequence takes 1 dysplay cell */
       b++;
       continue;
     }
-    b+= mblen;
+    b+= mb_len;
     pg= (wc >> 8) & 0xFF;
     clen+= utr11_data[pg].p ? utr11_data[pg].p[wc & 0xFF] : utr11_data[pg].page;
     clen++;
