@@ -46,6 +46,13 @@
 basedir=
 datadir=
 
+# Default value, in seconds, afterwhich the script should timeout waiting
+# for server start. 
+# Value here is overriden by value in my.cnf. 
+# 0 means don't wait at all
+# Negative numbers mean to wait indefinitely
+service_startup_timeout=900
+
 # The following variables are only set for letting mysql.server find things.
 
 # Set some defaults
@@ -126,6 +133,7 @@ parse_server_arguments() {
 	;;
       --user=*)  user=`echo "$arg" | sed -e 's/^[^=]*=//'` ;;
       --pid-file=*) server_pid_file=`echo "$arg" | sed -e 's/^[^=]*=//'` ;;
+      --service-startup-timeout=*) service_startup_timeout=`echo "$arg" | sed -e 's/^[^=]*=//'` ;;
       --use-mysqld_safe) use_mysqld_safe=1;;
       --use-manager)     use_mysqld_safe=0;;
     esac
@@ -142,24 +150,47 @@ parse_manager_arguments() {
 }
 
 wait_for_pid () {
+  verb="$1"
+  manager_pid="$2"  # process ID of the program operating on the pid-file
   i=0
-  while test $i -lt 900 ; do
-    sleep 1
-    case "$1" in
+  avoid_race_condition="by checking again"
+  while test $i -ne $service_startup_timeout ; do
+
+    case "$verb" in
       'created')
+        # wait for a PID-file to pop into existence.
         test -s $pid_file && i='' && break
-        kill -0 $2 || break # if the program goes away, stop waiting
         ;;
       'removed')
+        # wait for this PID-file to disappear
         test ! -s $pid_file && i='' && break
         ;;
       *)
-        echo "wait_for_pid () usage: wait_for_pid created|removed"
+        echo "wait_for_pid () usage: wait_for_pid created|removed manager_pid"
         exit 1
         ;;
     esac
+
+    # if manager isn't running, then pid-file will never be updated
+    if test -n "$manager_pid"; then
+      if kill -0 "$manager_pid" 2>/dev/null; then
+        :  # the manager still runs
+      else
+        # The manager may have exited between the last pid-file check and now.  
+        if test -n "$avoid_race_condition"; then
+          avoid_race_condition=""
+          continue  # Check again.
+        fi
+
+        # there's nothing that will affect the file.
+        log_failure_msg "Manager of pid-file quit without updating file."
+        return 1  # not waiting any more.
+      fi
+    fi
+
     echo $echo_n ".$echo_c"
     i=`expr $i + 1`
+    sleep 1
   done
 
   if test -z "$i" ; then
@@ -253,13 +284,13 @@ else
   esac
 fi
 
-# Safeguard (relative paths, core dumps..)
-cd $basedir
-
 case "$mode" in
   'start')
     # Start daemon
 
+    # Safeguard (relative paths, core dumps..)
+    cd $basedir
+
     manager=$bindir/mysqlmanager
     if test -x $libexecdir/mysqlmanager
     then
@@ -303,7 +334,7 @@ case "$mode" in
       fi
       exit $return_value
     else
-      log_failure_msg "Couldn't find MySQL manager or server"
+      log_failure_msg "Couldn't find MySQL manager ($manager) or server ($bindir/mysqld_safe)"
     fi
     ;;
 
@@ -327,7 +358,7 @@ case "$mode" in
       echo $echo_n "Shutting down MySQL"
       kill $mysqlmanager_pid
       # mysqlmanager should remove the pid_file when it exits, so wait for it.
-      wait_for_pid removed; return_value=$?
+      wait_for_pid removed "$mysqlmanager_pid"; return_value=$?
 
       # delete lock for RedHat / SuSE
       if test -f $lock_dir
@@ -351,20 +382,52 @@ case "$mode" in
     fi
     ;;
 
-  'reload')
+  'reload'|'force-reload')
     if test -s "$server_pid_file" ; then
-      mysqld_pid=`cat $server_pid_file`
+      read mysqld_pid <  $server_pid_file
       kill -HUP $mysqld_pid && log_success_msg "Reloading service MySQL"
       touch $server_pid_file
     else
       log_failure_msg "MySQL PID file could not be found!"
+      exit 1
     fi
     ;;
-
-  *)
-    # usage
-    echo "Usage: $0  {start|stop|restart|reload}  [ MySQL server options ]"
-    exit 1
+  'status')
+    # First, check to see if pid file exists
+    if test -s "$server_pid_file" ; then 
+      read mysqld_pid < $server_pid_file
+      if kill -0 $mysqld_pid 2>/dev/null ; then 
+        log_success_msg "MySQL running ($mysqld_pid)"
+        exit 0
+      else
+        log_failure_msg "MySQL is not running, but PID file exists"
+        exit 1
+      fi
+    else
+      # Try to find appropriate mysqld process
+      mysqld_pid=`pidof $sbindir/mysqld`
+      if test -z $mysqld_pid ; then 
+        if test "$use_mysqld_safe" = "0" ; then 
+          lockfile=/var/lock/subsys/mysqlmanager
+        else
+          lockfile=/var/lock/subsys/mysql
+        fi 
+        if test -f $lockfile ; then 
+          log_failure_msg "MySQL is not running, but lock exists"
+          exit 2
+        fi 
+        log_failure_msg "MySQL is not running"
+        exit 3
+      else
+        log_failure_msg "MySQL is running but PID file could not be found"
+        exit 4
+      fi
+    fi
+    ;;
+    *)
+      # usage
+      echo "Usage: $0  {start|stop|restart|reload|force-reload|status}  [ MySQL server options ]"
+      exit 1
     ;;
 esac
 
